#!/usr/bin/env python3
import click

from dataset_sh.server.app import create_app

from dataset_sh.server.config_cli import config_cli
from dataset_sh.server.core import RepoServerConfig


@click.group(name='dataset-sh-server')
def dsh_server():
    pass


cli = dsh_server


@click.command(name='start')
@click.option('--host', default='127.0.0.1', help='The interface to bind to.')
@click.option('--port', default=8989, type=int, help='The port to bind to.')
@click.option('--debug', is_flag=True, help='Run the server in debug mode.')
@click.option('--config', default=None, help='location of the dataset.sh server config file.')
def start(host, port, debug, config):
    server_config = None
    if config is not None:
        server_config = RepoServerConfig.load_from_file(config).override_from_env()
    app = create_app(config=server_config)
    app.run(host=host, port=port, debug=debug)


cli.add_command(start)
cli.add_command(config_cli)

if __name__ == '__main__':
    cli()



import json
import os.path
import time

import requests
from tqdm import tqdm

from dataset_sh.multipart.multipart import FilePartsInfo, read_file_range
from dataset_sh.utils.files import checksum

from requests.exceptions import HTTPError


class MultipartUploadClient(object):

    def __init__(self, url, source_file, headers=None, additional_params=None, progress_name=None):
        self.headers = headers
        self.source_file = source_file
        self.checksum = checksum(source_file)
        self.file_length = os.path.getsize(source_file)

        self.params = additional_params if additional_params is not None else {}
        self.params['checksum_value'] = self.checksum
        self.params['file_length'] = self.file_length
        self.url = url

        self.progress_name = progress_name

    def init_upload(self) -> FilePartsInfo:
        resp = requests.post(self.url, params={
            **self.params,
            'action': 'init'
        }, headers=self.headers)
        resp.raise_for_status()
        info = FilePartsInfo(**json.loads(resp.content.decode('utf-8')))
        return info

    def _upload_part(self, part_id, chunk_size):
        file_data = read_file_range(self.source_file, part_id * chunk_size, chunk_size)
        resp = requests.post(self.url, params={
            **self.params,
            'action': 'upload',
            'part_id': part_id,
        }, headers=self.headers, data=file_data)
        resp.raise_for_status()

    def _done(self):
        resp = requests.post(
            self.url, params={
                **self.params,
                'action': 'done'
            }, headers=self.headers)
        resp.raise_for_status()

    def upload(self, info=None):
        if info is None:
            info = self.init_upload()
        desc = f"Uploading {self.progress_name}" if self.progress_name else 'Upload Progress'
        for i in tqdm(
                range(info.parts_count), desc=desc, unit="it",
                bar_format="{l_bar}{bar}| {percentage:0.0f}% Completed"):
            if i not in info.finished:
                self._upload_part(i, info.chunk_size)
                time.sleep(2)
        self._done()

    def abort(self):
        resp = requests.post(
            self.url, params={
                **self.params,
                'action': 'abort'
            }, headers=self.headers)
        resp.raise_for_status()


def print_http_error(e):
    if e.response is not None:
        print(f"\n\nHTTP Status Code: {e.response.status_code}")
        if e.response.content:
            try:
                server_err = json.loads(e.response.content.decode('utf-8'))
                reason = server_err.get('reason', '')
                if reason:
                    print(f'UPLOAD FAILED because: {reason}\n\n')
            except json.JSONDecodeError as je:
                pass


def upload_to_url(upload_url, file_path, headers, params, name):  # pragma: no cover
    client = MultipartUploadClient(
        upload_url, file_path, headers=headers, additional_params=params, progress_name=name
    )

    upload_info = None
    try:
        upload_info = client.init_upload()
    except HTTPError as e:
        print(f'failed to init upload task.')
        print_http_error(e)
        raise e

    try:
        client.upload(info=upload_info)
    except HTTPError as e:
        print(f'upload failed, will try to clean up.')
        print_http_error(e)
        client.abort()



import json
import tempfile
from dataclasses import dataclass, field

from dataset_sh.clients import LocalStorage
from dataset_sh.utils.dump import dump_collections
import os


@dataclass
class DatasetVersions:
    versions: list[str] = field(default_factory=list)


@dataclass
class DatasetAndVersions:
    dataset: dict[str, DatasetVersions] = field(default_factory=dict)


TEST_USERNAME = 'test-user'
TEST_USERNAME_ANOTHER = 'test-another-user'


def create_test_datasets(location, count=10) -> DatasetAndVersions:
    lc = LocalStorage(location)
    ds = DatasetAndVersions()
    with tempfile.TemporaryDirectory() as scratch_folder:
        dataset_name = f'ad'
        fn = os.path.join(scratch_folder, 'tmp')

        vid = dump_collections(
            fn,
            {
                "seq": [
                    dict(name=f"ad-{i}", age=i) for i in range(15)
                ],
                "seq2": [

                ],
            }
            ,
        )

        lc.dataset(to_dataset_name(TEST_USERNAME_ANOTHER, dataset_name)).import_file(
            fn,
            verify_version=vid
        )

        for i in range(count):
            versions = []
            dataset_name = f'd{i}'
            for j in range(i):
                name = f'd-{i}-v-{j}'
                fn = os.path.join(scratch_folder, name)
                vid = dump_collections(
                    fn,
                    {
                        "seq": [
                            dict(name=f"d{i}", age=j)
                        ],
                    }
                    ,
                )
                versions.append(vid)
                lc.dataset(to_dataset_name(TEST_USERNAME, dataset_name)).import_file(
                    fn,
                    verify_version=vid
                )

            ds.dataset[dataset_name] = DatasetVersions(versions=versions)

            if len(versions) > 5:
                lc.dataset(to_dataset_name(TEST_USERNAME, dataset_name)).set_tag(
                    't1',
                    versions[-3]
                )
    return ds


def to_dataset_name(namespace, dataset_name):
    return f"{namespace}/{dataset_name}"


def sorted_list_of_dict(list1):
    return sorted(list1, key=lambda x: json.dumps(x))



import datetime
from dataclasses import dataclass
from typing import List, Optional

from pydantic import BaseModel, Field


@dataclass
class DatasetInfo:
    name: str
    is_redirect: bool


class DatasetInfoSnippet(BaseModel):
    namespace: str
    dataset: str


class NamespaceList(BaseModel):
    namespaces: List[str]


class DatasetListingResults(BaseModel):
    items: List[DatasetInfoSnippet]


class SourceInfo(BaseModel):
    url: str
    download_time: datetime.datetime = Field(default_factory=datetime.datetime.now)


class DatasetFileInternalPath:
    BINARY_FOLDER = 'bin'
    COLLECTION_FOLDER = 'coll'

    META_FILE_NAME = 'meta.json'
    DATA_FILE = 'data.jsonl'
    TYPE_FILE = 'type.json'


@dataclass
class DatasetSource:
    username: str
    dataset_name: str
    tag: Optional[str]
    version: str

    location: str



import os.path
import click

from dataset_sh.multipart.upload import MultipartUploadClient

file_size = 10 * 1024 * 1024  # 100MB


@click.command()
@click.option('--fn', default='test')
def run(fn):
    tmp_file = '/tmp/largefile'
    if not os.path.exists(tmp_file):
        with open(tmp_file, 'wb') as file:
            file.write(os.urandom(file_size))

    client = MultipartUploadClient(f'http://127.0.0.1:9900/upload/{fn}', tmp_file)
    client.upload()


if __name__ == '__main__':
    run()



SUPPORTED_TYPES = [
    'int',
    'float',
    'bool',
    'str',
    'list',
    'dict',
    'tuple',
    'set',
    'datetime.date',
    'datetime.datetime',
    'datetime.time',
    'datetime.timedelta',
    'typing.List',
    'typing.Set',
    'typing.Dict',
    'typing.Tuple',
    'typing.Union',
    'typing.Optional',
    'typing.Any',
]



import os
import datetime
from pydantic import BaseModel


class ArticleEntry(BaseModel):
    name: str
    mtime: str


class ProjectArticleManager:

    def __init__(self, base_folder):
        self.base_folder = os.path.normpath(base_folder)

    def list_articles(self, scan_folder: str = None) -> list[ArticleEntry]:
        if scan_folder is None:
            scan_folder = self.base_folder
        articles = []

        cat_file = os.path.join(self.base_folder, '__items__.txt')

        if os.path.exists(cat_file):
            with open(cat_file) as fd:
                for line in fd:
                    line = line.strip()
                    if line:
                        file_name = f"{line}.md"
                        file_path = os.path.join(self.base_folder, file_name)
                        if os.path.exists(file_path):
                            mtime = os.path.getmtime(file_path)
                            mtime_iso = datetime.datetime.fromtimestamp(mtime).isoformat()
                            name = file_name[:-3]
                            articles.append(ArticleEntry(name=name, mtime=mtime_iso))
        else:
            for file_name in os.listdir(scan_folder):
                if file_name.endswith('.md'):
                    file_path = os.path.join(self.base_folder, file_name)
                    mtime = os.path.getmtime(file_path)
                    mtime_iso = datetime.datetime.fromtimestamp(mtime).isoformat()
                    name = file_name[:-3]
                    articles.append(ArticleEntry(name=name, mtime=mtime_iso))
            articles.sort(key=lambda x: x.mtime, reverse=True)

        return articles

    def extract_article(self, file_name):
        file_path = os.path.join(self.base_folder, f"{file_name}.md")
        file_path = os.path.normpath(file_path)
        if not file_path.startswith(self.base_folder):
            return None
        if os.path.exists(file_path):
            with open(file_path, 'r', encoding='utf-8') as file:
                full_content = file.read()
                return full_content
        else:
            return None

    def write_article(self, file_name, content):
        file_path = os.path.join(self.base_folder, f"{file_name}.md")
        file_path = os.path.normpath(file_path)
        if not file_path.startswith(self.base_folder):
            return None

        os.makedirs(os.path.dirname(file_path), exist_ok=True)
        with open(file_path, 'w', encoding='utf-8') as out:
            out.write(content)



from typing import Union, Optional, List

import easytype.core as etc
from pydantic import BaseModel


class EnumEntry(BaseModel):
    name: str
    value: str


class FieldType(BaseModel):
    is_parametric: bool
    name: str
    children: Optional[List['FieldType']] = None

    @staticmethod
    def simple(name):
        return FieldType(is_parametric=False, name=name)

    @staticmethod
    def typed(name, children):
        return FieldType(is_parametric=True, name=name, children=children)

    def to_new_type_system(self) -> Union[etc.PrimitiveType, etc.ParameterizedType, str]:
        if self.is_parametric:
            params = [c.to_new_type_system() for c in self.name]
            ret = etc.ParameterizedType(name=self.name, params=params)
            return ret
        else:
            ret = etc.PrimitiveType
            return ret

    def get_types_to_build(self) -> List[str]:
        pass


class PydanticFieldDefinition(BaseModel):
    name: str
    type: FieldType

    def to_new_type_system(self) -> etc.FieldDefinition:
        ret = etc.FieldDefinition(
            name=self.name,
            type=self.type.to_new_type_system(),
            comment=None,
        )
        return ret

    def get_types_to_build(self) -> List[str]:
        pass


class EnumDefinition(BaseModel):
    name: str
    enum_entries: Optional[List[EnumEntry]]
    doc_str: Optional[str] = None

    def is_int(self):
        if self.enum_entries is not None and len(self.enum_entries) > 0:
            return isinstance(self.enum_entries[0].value, int)
        return False

    def to_new_type_system(self) -> etc.EnumType:
        ret = etc.EnumType(
            name=self.name,
            choices=[x.value for x in self.enum_entries],
            comment=self.doc_str
        )
        return ret

    def get_types_to_build(self) -> List[str]:
        pass


class ClassDefinition(BaseModel):
    name: str
    fields: List[PydanticFieldDefinition]
    doc_str: Optional[str] = None

    def to_new_type_system(self) -> etc.UserDefinedType:
        return etc.UserDefinedType(
            name=self.name,
            fields=[x.to_new_type_system() for x in self.fields],
            comment=self.doc_str
        )

    def get_types_to_build(self) -> List[str]:
        pass


class DatasetSchema(BaseModel):
    entry_point: str
    classes: List[Union[EnumDefinition, ClassDefinition]]

    def to_new_type_system(self) -> etc.UserDefinedType:
        def_lookup = {}
        build_order = self._get_build_order()
        for c in self.classes:
            def_lookup[c.name] = c.to_new_type_system()
        etc.UserDefinedType(name='', comment='')



from typing import List, Optional
from pydantic import BaseModel, Field


class DatasetDependencyItem(BaseModel):
    """
    e.g.
        * user/dataset
        * user/dataset:tag=latest
        * user/dataset:
        * user/dataset
        * user/dataset
    Both source and target should be able to parsed by a DatasetLocator.
    """
    source: str
    target: Optional[str] = None


class DatasetDependencyHostGroup(BaseModel):
    host: str
    datasets: List[DatasetDependencyItem] = Field(default_factory=list)


class DatasetDependencies(BaseModel):
    dependencies: List[DatasetDependencyHostGroup] = Field(default_factory=list)



from typing import Optional

from easytype.core import UserDefinedType

from dataset_sh import create
from dataset_sh.utils.files import checksum
from dataset_sh.utils.misc import get_tqdm
from dataset_sh.constants import DEFAULT_COLLECTION_NAME


def dump_single_collection(
        fn,
        data,
        name=DEFAULT_COLLECTION_NAME,
        type_annotation: Optional[UserDefinedType] = None,
        silent=False
):
    """
    Args:
        fn:
        data:
        name:
        type_annotation:
        silent:
    Returns:
    """
    with create(fn) as out:
        out.add_collection(name, data, type_annotation=type_annotation, tqdm=get_tqdm(silent=silent))
    return checksum(fn)


def dump_collections(
        fn,
        data_dict,
        type_dict=None,
        report_item_progress=False,
        report_collection_progress=False
):
    """
    Args:
        fn:
        data_dict:
        type_dict:
        report_item_progress:
        report_collection_progress:

    Returns:

    """
    if type_dict is None:
        type_dict = {}
    inner_tqdm = get_tqdm(silent=not report_item_progress)
    with create(fn) as out:
        for name, data in data_dict.items():
            if report_collection_progress:
                print(f'Importing collection {name}')
            if len(data) > 0:
                type_annotation = type_dict.get(name, None)
                out.add_collection(name, data, type_annotation=type_annotation, tqdm=inner_tqdm)
    return checksum(fn)
